const fs = require('fs');
const path = require('path');
const webpack = require('webpack');
const VueLoaderPlugin = require('vue-loader/lib/plugin');
const vueLoaderConfig = require('./vue-loader.conf');
const { _config } = require('./getBlConfig');
const jsonConditionalCompilation = require('./jsonConditionalCompilation');

module.exports = (ROOT, SRC, mode = 'development') => {
    const DIST = path.join(ROOT, _config, 'dist');

    fs.writeFileSync(path.resolve('service.js'), '', 'utf8');

    let blConfig = { isUni: true };

    if (fs.existsSync(path.join(ROOT, 'bl.config.js'))) {
        blConfig = require(path.join(ROOT, 'bl.config.js'));
        if (blConfig.isUni === undefined) {
            blConfig.isUni = true;
        }
    }

    let appJson = fs.readFileSync(path.join(SRC, 'app.json'), 'utf8');
    let _jsonData = JSON.parse(appJson);
    appJson = jsonConditionalCompilation({
        content: appJson,
        platform: 'bl'
    });
    const playletInclude = [];
    _jsonData.pages.forEach(page => {
        if(page.style && 
            page.style.extends === 'ext://industry/playlet-plugin' && 
            page.style.isPageExtension === true){
            playletInclude.push(path.join(ROOT, 'src', page.path));
        }
    });
    const configs = {
        node: {
            fs: 'empty'
        },
        mode: mode,
        entry: {
            service: './service.js',
        },
        output: {
            globalObject: 'this',
            filename: 'service.js',
            path: DIST,
        },
        module: {
            rules: [
                {
                    test: /node_modules\/three/,
                    use: [
                        {
                            loader: path.resolve(
                                __dirname,
                                "./loaders/threejs-loader"
                            ),
                        },
                    ],
                },
                {
                    test: /\.vue$/,
                    use: [
                        {
                            loader: path.resolve(__dirname, "./loaders/uni-vue-post-loader"),
                        },
                        {
                            loader: path.resolve(
                                __dirname,
                                "./loaders/bl-sapp-service-post-loader"
                            ),
                        },
                        {
                            loader: "vue-loader",
                            options: vueLoaderConfig,
                        },
                        {
                            loader: path.resolve(
                                __dirname,
                                "./loaders/bl-sapp-service-pre-loader"
                            ),
                        },
                        { 
                            loader: path.resolve(__dirname, './loaders/bl-sapp-extends-loader'),
                            options: {
                                include: playletInclude
                            },
                            
                        },
                        // { loader: 'vue-blservice-loader' },
                        // {
                        //     loader: 'uni-vue-loader',
                        //     options: {
                        //         platform: 'bl',
                        //         isUni: blConfig.isUni
                        //     }
                        // }
                    ],
                },
                {
                    test: /\.js$/,
                    use: [
                        {
                            loader: "babel-loader",
                        },
                        {
                            loader: "uni-js-loader",
                            options: {
                                platform: "bl",
                            },
                        },
                    ],
                    include: [
                        SRC,
                        path.join(ROOT, "node_modules", "bili-small-app", "runtime"),
                        path.join(ROOT, "node_modules", "bili-small-app", "components"),
                    ],
                },
                {
                    test: /\.ts$/,
                    use: [
                        {
                            loader: "babel-loader",
                        },
                        {
                            loader: "uni-js-loader",
                            options: {
                                platform: "bl",
                            },
                        },
                        {
                            loader: "ts-loader",
                            options: {
                                transpileOnly: true,
                                appendTsSuffixTo: ["\\.vue$"],
                            },
                        },
                    ],
                },
                {
                    test: /\.(png|jpe?g|gif|svg)$/,
                    loader: "null-loader",
                },
                {
                    test: /service\.js$/,
                    use: [
                        { loader: "babel-loader" },
                        { loader: "bili-small-app/build/service-loader" },
                    ],
                },
                {
                    test: /small-app[/\\]runtime[/\\]bl/,
                    use: [
                        { loader: "babel-loader" },
                        {
                            loader: "string-replace-loader",
                            options: {
                                search: "__BLJS__",
                                replace: "import bl from './bl.app'",
                                flags: "g",
                            },
                        },
                    ],
                },
                {
                    test: /\.pug$/,
                    oneOf: [
                        // 这条规则应用到 Vue 组件内的 `<template lang="pug">`
                        {
                            resourceQuery: /^\?vue/,
                            use: [
                                {
                                    loader: path.resolve(
                                        __dirname,
                                        "./loaders/uni-vue-post-loader"
                                    ),
                                },
                                "pug-plain-loader",
                            ],
                        },
                        // 这条规则应用到 JavaScript 内的 pug 导入
                        {
                            use: ["raw-loader", "pug-plain-loader"],
                        },
                    ],
                },
                {
                    test: /\.json$/,
                    use: [
                        {
                            loader: "uni-json-loader",
                            options: {
                                platform: "bl",
                            },
                        },
                    ],
                },
                {
                    test: /\.css$/,
                    use: ["null-loader"],
                },
                {
                    test: /\.less$/,
                    use: ["null-loader"],
                },
                {
                    test: /\.styl$/,
                    use: ["null-loader"],
                },
            ],
        },
        resolve: {
            extensions: ['.js', '.vue', '.json', '.styl'],
            alias: {
                '@': ROOT,
                '%': SRC
            },
            mainFields: ['main']
        },
        plugins: [
            new VueLoaderPlugin(),
            new webpack.DefinePlugin({
                GLOBAL_OBJECT: 'Service',
                IS_SMALL_GAME: 'false',
                'process.env': require('./config').env,
                APPID: '"' + JSON.parse(fs.readFileSync(path.join(ROOT, _config, '.sapprc'), 'utf8')).appId + '"',
                APP_JSON: appJson
                // IS_DEV: String(mode === 'development'),
                // IP: '"' + getIpAddress() + '"'
            })
        ],
        externals: {
            vue: 'Vue'
        }
    };
    if (process.env.BL_NOMIN) {
        configs.optimization = {
            minimize: false
        };
    }
    return configs;
};
