import { EventEmitter } from "events";
import { callNative,callNativeSync } from "./util";

const globalEventBus = new EventEmitter();
// 统一的支付弹窗状态控制
let toggleCustomDialogValue = false;
// 选集面板弹窗展示状态
let toggleEpisodeDialogValue = false;

globalEventBus.on("closeCustomDialogHandle", (data) => {
    PlayletManager.playletManagerMap[data.id].toggleCustomDialog(
        "close",
        "service"
    );
});
globalEventBus.on("setEpisodeSelected", (data) => {
    toggleEpisodeDialogValue = false;
    // 传递选集参数
    if(data.options.seq){
        callNative("setEpisodeSelected", { pageId: data.pageId, options: data.options });
    }
    // 关闭选集面板
    callNative("toggleEpisodeDialog", { pageId: data.pageId, options: { type: "close" }});
});
// 插件被销毁时，清除全局事件监听
globalEventBus.on("playletPluginDestroyed", () => {
    PlayletManager.playletManagerMap = {};
    isPlayletManagerCreated = false;
    toggleEpisodeDialogValue = false;
    toggleCustomDialogValue = false;
});
class PlayletManager {
    constructor(context, pageId, id) {
        this.context = context;
        this.callbacks = {};
        Object.defineProperties(this, {
            toggleCustomDialogValue: {
                value: false,
                writable: true,
                enumerable: false,
                configurable: false,
            },
            _pageId: {
                value: pageId,
                writable: false,
                enumerable: false,
                configurable: false,
            },
            _id: {
                value: '',
                writable: true,
                enumerable: false,
                configurable: false,
            },
        });

        this._id = id;
        PlayletManager.playletManagerMap[this._id] = this;
    }

    /**
   * 设置分享信息和自定义icon
   * @param {Object} options - 配置选项
   * @param {Object} [options.shareParam] - 小程序分享参数
   * @param {string} [options.shareParam.title] - 要转发的小程序标题
   * @param {string} [options.shareParam.desc] - 默认的转发文案
   * @param {string} [options.shareParam.path] - 转发页面路径
   * @param {string} [options.shareParam.imageUrl] - 图片URL
   * @param {string} [options.shareParam.templateId] - 分享素材模板ID
   * @param {Array} [options.activityInfo] - 自定义按钮配置
   * @param {string} [options.activityInfo.icon] - 自定义icon的图片
   * @param {string} [options.activityInfo.title] - 自定义icon的标题
   * @param {boolean} [options.showLockPage=true] - 是否展示解锁页
   * @param {'contain' | 'cover' | 'fill'} [options.objectFit] - 视频展现形式，竖屏场景默认为cover，横屏场景默认为contain
   * @param {string} [options.playbackRate] - 播放倍速 0.75 | 1.0 | 1.25 | 1.5 | 2
   */
    setConfig(options = {}) {
        options.showLockPage = true;
        callNative("setConfig", { options, pageId: this._pageId });
    }

    /**
   * 获取当前剧目信息
   * @returns {Promise<Object>} - 返回包含剧目信息的Promise对象
   * @returns {string} albumId - 剧ID
   * @returns {number} seq - 集数
   * @returns {string} episodeId - 集ID
   */
    getPlayletInfo() {
        return new Promise((resolve, reject) => {
            callNative("getPlayletInfo", {
                pageId: this._pageId,
                success: function(res) {
                    // console.log("[callNative]: getPlayletInfo-success", res);
                    const systemInfo = callNativeSync("getSystemInfoSync").data;
                    if (systemInfo.platform === "ios") {
                        res.episodeId = res.episode_id;
                    }
                    delete res.errMsg;
                    // delete res.episode_id;
                    resolve(res);
                },
                fail: function(e) {
                    // console.log("[callNative]: getPlayletInfo-fail", e);
                    reject(e);
                },
            });
        });
    }
    
    /**
   * 更新目录设置
   * @param {Object} options - 配置选项
   * @param {Array} [options.lockList] - 已锁定的页面范围
   * @param {Array} [options.unlockList] - 已解锁的页面范围
   * @param {Array} [options.freeList] - 免费的页面范围
   */
    setCatalog(options) {
        callNative("setCatalog", { options, pageId: this._pageId });
    }

    /**
   * 打开/关闭自定义模块
   * @param {'open' | 'close'} [type] - 模块状态
   */
    toggleCustomDialog(type, source) {
        toggleCustomDialogValue = type
            ? type === "open"
            : source
                ? false
                : !toggleCustomDialogValue;
        if (!source) {
            globalEventBus.emit("toggleCustomDialog", {
                type: toggleCustomDialogValue,
                pageId: this._pageId,
                id: this._id,
            });
        }
        const options = {
            type: type ? type : toggleCustomDialogValue ? "open" : "close",
        };
        // 选集弹窗展示时，关闭选集弹窗
        if(toggleEpisodeDialogValue){
            globalEventBus.emit("closeCatalogDialog", {
                catalogData: {},
                type: false,
                pageId: this._pageId,
                id: this._id,
            });
            callNative("toggleEpisodeDialog", { pageId: this._pageId, options: { type: "close" } });
        }
        
        setTimeout(() => {
            callNative("toggleCustomDialog", { options, pageId: this._pageId });
        }, 0);
    }

    /**
   * 设置当前集为解锁
   */
    setCurrentUnlock() {
        callNative("setCurrentUnlock", { pageId: this._pageId });
    }

    /**
   * 设置暂停/播放状态
   * @param {'play' | 'pause'} status - 视频状态
   */
    setPlayStatus(status) {
        if (!status) {
            throw new Error("setPlayStatus: status is required");
        }
        if (status !== "play" && status !== "pause" && status !== "replay") {
            throw new Error('setPlayStatus: status must be "play" or "pause" or "replay"');
        }
        callNative("setPlayStatus", { options: { status }, pageId: this._pageId });
    }
    /**
     * 设置推荐位
     */
    setRecommendConfig(options) {
        callNative("setRecommendConfig", { options, pageId: this._pageId });
    }

    /**
     * 设置试看时长
     */
    setUnlockPreview(options){
        const {second , episodeConfig} = options;
        if(!second && !episodeConfig){
            throw new Error("setUnlockPreview: second or episodeConfig is required");
        }
        const failCallback = (error) => {
            this.trigger("onError", error);
        };
        callNative("setUnlockPreview", { options, pageId: this._pageId , fail: failCallback});
    }

    /**
   * 设置试看按钮
   * @param {string} id - 要配置的按钮id，setUnlockPreview的buttons里配置的id
   * @param {Object} config - 配置的按钮内容
   * @param {string} config.text - 按钮文案，字符长度4～10（包含标点符号、空格）
   */
    setUnlockPreviewButton(id, config) {
        if (!id) {
            throw new Error("setUnlockPreviewButton: id is required");
        }

        if (!config || !config.text) {
            throw new Error("setUnlockPreviewButton: config.text is required");
        }

        const failCallback = (error) => {
            this.trigger("onError", error);
        };

        callNative("setUnlockPreviewButton", {
            options: { id, config},
            pageId: this._pageId,
            fail: failCallback 
        });
    }
    
    /**
     * 用户设置选集
     * @param {Object} options - 配置选项
     * @param {string} options.seq - 集
     */
    setEpisodeSelected(options){
        callNative("setEpisodeSelected", { pageId: this._pageId, options });
    }

    /**
   * 注册事件回调
   * @param {string} event - 事件名称
   * @param {Function} callback - 回调函数
   */
    on(event, callback) {
        if (!this.callbacks[event]) {
            this.callbacks[event] = [];
        }
        this.callbacks[event].push(callback);
    }

    /**
   * 触发事件
   * @param {string} event - 事件名称
   * @param {Object} data - 事件数据
   */
    trigger(event, data) {
        if (this.callbacks[event]) {
            this.callbacks[event].forEach((callback) => callback.call(this, data));
        }
    }

    // 注册事件回调函数
    onPlay(callback) {
    // todo 插件页面的onPlay先触发， 在触发player插件的onPlay
        this.on("onPlay", callback);
    }

    onPause(callback) {
        this.on("onPause", callback);
    }

    onEnded(callback) {
        this.on("onEnded", callback);
    }

    onError(callback) {
        this.on("onError", callback);
    }

    onTimeUpdate(callback) {
        this.on("onTimeUpdate", callback);
    }

    onProgress(callback) {
        this.on("onProgress", callback);
    }

    onWaiting(callback) {
        this.on("onWaiting", callback);
    }

    onPlayBackRateChange(callback) {
        this.on("onPlayBackRateChange", callback);
    }

    onLoadedMetaData(callback) {
        this.on("onLoadedMetaData", callback);
    }
    // 点击控件-控件类型: 
    // [play, pause, playbackRate, progress, like, unlike, share, subscribe, unsubscribe, openCatalog, closeCatalog, userCloseCharge, openRatePanel, closeRatePanel]
    // [播放, 暂停, 在倍速选择面板点击具体速度, 进度控件，在刚开始拖动进度条时触发 , 点击喜欢, 点击不喜欢, 分享, 点击追剧, 点击取消追剧, 点击打开目录页, 点击关闭目录页, 用户点击蒙层关闭充值页, 点击打开播放速度选择面板, 点击关闭播放速度选择面板]
    onControlTap(callback) {
        this.on("onControlTap", callback);
    }
    controlTapFn(message) {
        // data 包含选集信息
        const { data } = message;
        if(!data || !data.album_info){
            throw new Error("onSelectEpisodeClick: no data or albumInfo");
        }
        // 渲染选集面板弹窗
        globalEventBus.emit("openCatalogDialog", {
            catalogData: data,
            type: true,
            pageId: this._pageId,
            id: this._id,
        });
        toggleEpisodeDialogValue = true;
        // 调用客户端 API 展示面板
        if(toggleCustomDialogValue){
            callNative("toggleCustomDialog", { pageId: this._pageId, options: { type: "close" } });
        }
        callNative("toggleEpisodeDialog", { pageId: this._pageId, options: { type: "open" } });

    }
    onOpenCatalog(callback) {
        this.on("onOpenCatalog", callback);
    }

    onChangeEpisode(callback) {
        this.on("onChangeEpisode", callback);
    }

    onClickUnlock(callback) {
        this.on("onClickUnlock", callback);
    }

    onTapShare(callback) {
        this.on("onTapShare", callback);
    }

    onShareSuccess(callback) {
        this.on("onShareSuccess", callback);
    }

    onShareFail(callback) {
        this.on("onShareFail", callback);
    }

    onTapCustomIcon(callback) {
        this.on("onTapCustomIcon", callback);
    }

    onAddShortCut(callback) {
        this.on("onAddShortCut", callback);
    }

    onMuteChange(callback) {
        this.on("onMuteChange", callback);
    }

    onSeekComplete(callback) {
        this.on("onSeekComplete", callback);
    }
    // 试看结束回调
    onPreviewEnd(callback) {
        this.on("onPreviewEnd", callback);
    }

    // 按钮点击事件
    onUnlockPreviewButtonClick(callback) {
        this.on("onUnlockPreviewButtonClick", callback);
    }
}
PlayletManager.playletManagerMap = {};
function createPlayletContext(pageId) {
    return new Promise((resolve, reject) => {
        callNative("createPlayletContext", {
            pageId,
            success: function(res) {
                // console.log("[callNative]: createPlayletContext-success", res);
                resolve(res);
            },
            fail: function(e) {
                // console.log("[callNative]: createPlayletContext-fail", e);
                reject(e);
            },
        });
    });
}
const playletMessageQueue = [];
let isPlayletManagerCreated = false;
/**
 * 获取 PlayletManager 实例
 * @param {Object} context - 上下文对象
 * @returns {PlayletManager} - PlayletManager 实例
 */
async function getPlayletManager(context) {
    // 组件创建实力时会传入 参数 {is: '组件名‘}，要等待页面创建完成之后，在执行创建，否则会有 401 报错
    // console.log("[service:]getPlayletManager", context ,"isPlayletManagerCreated",isPlayletManagerCreated);
    const currentPageId = global.__biliCacheData.getCurrentPageId();
    if(context&&context.is){
        if (!isPlayletManagerCreated) {
            // 将消息存储在队列中
            return new Promise((resolve) => {
                playletMessageQueue.push({ context, resolve });
            });
        }
    }
    const data = await createPlayletContext(currentPageId);
    // console.log("[service:]createPlayletContext", data.id);
    if (data.id || data.id === 0) {
        isPlayletManagerCreated = true;
        const manager = new PlayletManager(context, currentPageId, data.id);
        // 处理队列中的消息
        while (playletMessageQueue.length > 0) {
            const { context, resolve } = playletMessageQueue.shift();
            getPlayletManager(context).then(resolve);
        }
        return manager;
    }
    return data;
}
// 处理客户端事件回调
function playletPluginHandleMessage(message) {
    // console.log("[service:]playletPluginHandleMessage", message);
    const playlet = PlayletManager.playletManagerMap[message.id];
    if (playlet) {
        // 特殊处理选集回调
        if(message.event === "onSelectEpisodeClick"){
            playlet.controlTapFn(message);
            return;
        }
        if(message.event === "onChangeEpisode" && message.data.scene !== 'first_play' && toggleEpisodeDialogValue){
            const data = {
                season_id: message.data.albumId,
                ord: message.data.seq,
            };
            globalEventBus.emit("openCatalogDialog", {
                catalogData: data,
                type: true,
                pageId: playlet._pageId,
                id: playlet._id,
            });
        }
        playlet.trigger(message.event, message.data || {});
    }
}

export { PlayletManager, playletPluginHandleMessage };
export default {
    getPlayletManager,
    globalEventBus,
};
